% GMM estimation of Fama-MacBeth Regressions
% Modified Berg and Mark (2016) code

function [o1,o2,o3,o4,B] = BergMarkFMB(Re,f)
   
wc = 0;          % 0: No constant 1: with constant
nwlag = 2;

nret = size(Re,2);
bigbigT = size(Re,1);
    
% Estimation begins here   
    nf = size(f,2);    
    for ret = 1 : nret; % For each return, do Time Series regression with all available obs.
        design = [Re(:,ret) f];   % design matrix
        nanMap = isnan(design);   % Elminate missing observations
        for j = 1 : size(design,2);
            tt0x = find(nanMap(:,j) == 0,1,'first');
            tt1x = find(nanMap(:,j) == 0,1,'last');
            if j == 1;
                tt0 = tt0x;
                tt1 = tt1x;
            else
                tt0 = [tt0 tt0x];
                tt1 = [tt1 tt1x];
            end
        end
        t0 = max(tt0);    % Balanced sample of return and factor
        t1 = min(tt1);  
        if ret == 1;       % Create a record of samples
            sto_t0 = t0;
            sto_t1 = t1;
        else
            sto_t0 = [sto_t0; t0];
            sto_t1 = [sto_t1; t1];
        end
        t0 = max(sto_t0);
        t1 = min(sto_t1);
        T = t1 - t0 + 1;
    end
    
    % Time series regression. Betas estimated on balanced panel
    for ret = 1 : nret;
        y = Re(t0:t1,ret);
        x = [ones(T,1) f(t0:t1,:) ];
        b = x \ y;

        tserror(t0:t1,ret) = y - x*b;  % For Shanken adjustment
        betatil(:,ret) = b;            % For GMM standard errors (includes the constant)
        a(1,ret) = b(1,1);             % Constant in the time-series regression
        beta(:,ret) = b(2:end,1);      % Vector of betas for each return (column)
    end
    B = beta';                         % Does not include constant from time series regression
   
    
    % One cross-sectional regression to estimate lambda. For each asset, compute the time-series average
    sto = Re;
    nanMap = isnan(sto);
    for i = 1 : nret;
        rbar(i,1) = mean(sto(t0:t1,i));
    end
    y = rbar;
    nmom = nret*(nf+2);
    if wc == 0;     % Cross-sectional regression without constant
        x = beta'; 
        lambda = x \ y;
        alpha = y - x * lambda;
        Rsq = 1 - alpha'*alpha/((y)'*(y));
        pred = x*lambda;
        % GMM standard errors
        theta = [betatil(:,1)];  % Stacking the betas. Includes constant from the time series regressions
        for jj = 2 : nret;
            theta = [theta ; betatil(:,jj)];
        end
        
        betadim = size(theta,1);
        theta = [ theta; lambda];
        M_F = zeros(nf+1,nf+1);
        ftil = zeros(nf + 1, bigbigT);
        for t = t0 : t1;
            ftil(:,t) = [1 ; f(t,:)'];
            M_F = M_F + ftil(:,t)*ftil(:,t)';
        end
        M_F = M_F / T ;

        u = zeros(nmom,bigbigT);
        for t = t0 : t1;
            uu = zeros(nf + 1);
            for i = 1 : nret;
                if i == 1;
                    uu = ftil(:,t) * (Re(t, i) - ftil(:,t)' * betatil(:,i)); 
                else
                    uu = [uu ; ftil(:,t) * (Re(t, i) - ftil(:,t)' * betatil(:,i))];
                end
            end
            uu = [uu ; Re(t,:)' - B * lambda];
            u(:,t) = uu;
        end
        % S matrix (spectral density matrix of u at frequency zero)
        ft = u(:,t0:t1); 
        nmom = size(ft,1);
        T = size(ft,2);        
        w = zeros(nmom,nmom);
        for t = 1 : T;
            w = ft(:,t)*ft(:,t)' + w;
        end
        s = w;
        for i = 1 : nwlag;
            w = zeros(nmom,nmom);
            for t = 1 : T - i;
                w = ft(:,t)*ft(:,t+i)' + w;
            end
            s = s + (w + w')* (1 - i/(nwlag + 1));
        end
        ST = s/T;       % Spectral density matrix

        Mat1 = -kron(eye(nret),M_F);
        Mat2 = zeros(nret*(nf+1),nf);
        tmp0 = [ 0 lambda'];
        Mat3 = -kron(eye(nret),tmp0);
        Mat4 = -B;
        dT = [Mat1 Mat2; Mat3 Mat4];
        
        aT = [eye(nret*(nf+1)) zeros(nret*(nf+1),nret) ; zeros(nf,nret*(nf+1)) B'];
        V_theta = pinv(aT*dT) *(aT * ST * aT') *(pinv(aT *dT))' / T ;
        GMM_T_theta = theta ./ sqrt(diag(V_theta));
        GMM_T_lambda = GMM_T_theta(betadim + 1:end);  
        
        Mat1 = eye(nret*(nf + 2)) - dT * pinv(aT * dT) * aT;
        V_g = Mat1 * ST * Mat1'/T;
        
        GMM_Sig_alpha = V_g(nmom - (nret-1) : end, nmom - (nret-1): end);
        GMM_pe_test  = alpha'*pinv(GMM_Sig_alpha)*alpha;          % GMM pricing error test statistic
        GMM_pval_pe_test = 1-chi2cdf( GMM_pe_test,nret-1);
                
        
       figure;
           bb = y*inv(y'*y)*y'*pred;
        plot1 = plot(y,[pred, bb]);
        
        set(plot1(1),'Marker','o','LineStyle','none');
        xlabel('Realized Returns');
        ylabel('Predicted Reurns');
        
    end
    
    if wc == 1;
        x = [ones(nret,1) beta'];  
        coeffs = x \ y;            % Cross-sectional regression
        alpha = y - x*coeffs;      % Pricing error
        pred = x*coeffs;           % predicted part for graphs
        Rsq = 1 - alpha'*alpha/((y-mean(y))'*(y-mean(y)));
        
        gam = coeffs(1);        % Cross-sectional constant
        lambda = coeffs(2:end); % Price of risk
        
        
  % Scatter plot the betas against mean returns. Returns on horizontal axis
  
   figure;
           y0 = [y, ones(size(y,1),1)];
           bb = y0*inv(y0'*y0)*y0'*pred;
        plot1 = plot(y,[pred, bb]);      
        set(plot1(1),'Marker','o','LineStyle','none');
        xlabel('Realized Returns');
        ylabel('Predicted Reurns');
 
   
    %------------------------------------------------------------------
         
        % GMM standard errors
        theta = [betatil(:,1)];  % Stack the betas. Includes constant from time-series regression
        for jj = 2 : nret;
            theta = [theta ; betatil(:,jj)];
        end
        
        betadim = size(theta,1);
        theta = [ theta; gam; lambda];  % Add cross-sectional constant and lambda
        X = [ones(nret,1) B];           % Augmentation for the constant

        M_F = zeros(nf+1,nf+1);
        ftil = zeros(nf + 1, bigbigT);
        for t = t0 : t1;
            ftil(:,t) = [1 ; f(t,:)'];
            M_F = M_F + ftil(:,t)*ftil(:,t)';
        end
        M_F = M_F / T ;
        
        u = zeros(nmom,bigbigT);       % GMM moment conditions
        for t = t0 : t1;
            uu = zeros(nf + 1);
            for i = 1 : nret;
                if i == 1;
                    uu = ftil(:,t) * (Re(t, i) - ftil(:,t)' * betatil(:,i)); 
                else
                    uu = [uu ; ftil(:,t) * (Re(t, i) - ftil(:,t)' * betatil(:,i))];
                end
            end
            sto = [gam ; lambda];
            uu = [uu ; Re(t,:)' - X * sto];
            u(:,t) = uu;
        end
        % S matrix (spectral density matrix of u at frequency zero)
        ft = u(:,t0:t1); 
        nmom = size(ft,1);
        T = size(ft,2);        
        w = zeros(nmom,nmom);
        for t = 1 : T;
            w = ft(:,t)*ft(:,t)' + w;
        end
        s = w;
        for i = 1 : nwlag;
            w = zeros(nmom,nmom);
            for t = 1 : T - i;
                w = ft(:,t)*ft(:,t+i)' + w;
            end
            s = s + (w + w')* (1 - i/(nwlag + 1));
        end
        ST = s/T;       % Spectral density matrix

        Mat1 = -kron(eye(nret),M_F);
        Mat2 = zeros(nret*(nf+1),(nf+1));
        tmp0 = [ 0 lambda'];
        Mat3 = -kron(eye(nret),tmp0);
        Mat4 = -X;
        dT = [Mat1 Mat2; Mat3 Mat4];
        
        aT = [eye(nret*(nf+1)) zeros(nret*(nf+1),nret) ; zeros((nf+1),nret*(nf+1)) X'];
        V_theta = pinv(aT*dT) *(aT * ST * aT') *(pinv(aT *dT))' / T ;
        GMM_T_theta = theta ./ sqrt(diag(V_theta));
        GMM_T_lambda = GMM_T_theta(betadim + 1:end); 

        Mat1 = eye(nret*(nf + 2)) - dT * pinv(aT * dT) * aT;
        V_g = Mat1 * ST * Mat1'/T;
        
        GMM_Sig_alpha = V_g(nmom - (nret-1) : end, nmom - (nret-1): end);
        GMM_pe_test  = alpha'*pinv(GMM_Sig_alpha)*alpha;   % GMM pricing error test statistic
        GMM_pval_pe_test = 1-chi2cdf( GMM_pe_test,nret-1);
   

    end
   
    if wc == 1;
                   o1 = [gam, lambda'];      
    else
                   o1 = lambda';   
    end
    
    o4 = [ GMM_pe_test  GMM_pval_pe_test];
    o2 = GMM_T_lambda';
    o3 = Rsq;
  
    
